import Zope
import unittest

import FunctionalTests
from FunctionalTests.Framework import *

from Globals import package_home
import os, string

class FauxInvocation:

    def __init__( self, result, elapsed ):
        self._result    = result
        self._elapsed   = elapsed

    def getResult( self ):
        return self._result

    def getElapsedTime( self ):
        return self._elapsed

class FauxRequest:

    def __init__( self, name, result, time ):
        self._name      = name
        self._result    = result
        self._time      = time

    def getName( self ):
        return self._name

    def getExpectedResult( self ):
        return self._result

    def getExpectedTime( self ):
        return self._time

MARKER = []
MARKER2 = []
class ResultTests( unittest.TestCase ):

    def setUp( self ):
        pass

    def tearDown( self ):
        pass

    def test_empty( self ):

        empty = Result( test=MARKER )

        assert empty.getTest() is MARKER
        assert empty.timeRequests()
        assert empty.checkResponses()
        assert empty.checkElapsedTimes()
        assert not empty.listInvocations()
        assert not empty.listErrors()
        assert not empty.listErrors( fatal_only=1 )
        assert not empty.listChildren()

    def test_apathy( self ):

        apathy = Result( test=MARKER
                       , time_requests=0
                       , check_responses=0
                       , check_elapsed_times=0
                       )

        assert apathy.getTest() is MARKER
        assert not apathy.timeRequests()
        assert not apathy.checkResponses()
        assert not apathy.checkElapsedTimes()

        apathy.addInvocation( FauxInvocation( 500, 3600.0 )
                            , FauxRequest( 'foo', 200, 1.0 )
                            )
        self.assertEquals( len( apathy.listInvocations() ), 1 )
        assert not apathy.listErrors()
        assert not apathy.listErrors( fatal_only=1 )
        assert not apathy.listChildren()

    def test_responseOnly( self ):

        driven = Result( test=MARKER
                       , time_requests=0
                       , check_responses=1
                       , check_elapsed_times=0
                       )

        assert driven.getTest() is MARKER
        assert not driven.timeRequests()
        assert driven.checkResponses()
        assert not driven.checkElapsedTimes()

        driven.addInvocation( FauxInvocation( 500, 3600.0 )
                            , FauxRequest( 'foo', 200, 1.0 )
                            )
        self.assertEquals( len( driven.listInvocations() ), 1 )
        self.assertEquals( len( driven.listErrors() ), 1 )
        self.assertEquals( len( driven.listErrors( fatal_only=1 ) ), 1 )
        assert not driven.listChildren()

    def test_timeOnly( self ):

        consistent = Result( test=MARKER
                           , time_requests=0
                           , check_responses=0
                           , check_elapsed_times=1
                           )

        assert consistent.timeRequests()

        impatiens = Result( test=MARKER
                          , time_requests=1
                          , check_responses=0
                          , check_elapsed_times=1
                          )

        assert impatiens.getTest() is MARKER
        assert consistent.timeRequests()
        assert not impatiens.checkResponses()
        assert impatiens.checkElapsedTimes()

        impatiens.addInvocation( FauxInvocation( 500, 3600.0 )
                            , FauxRequest( 'foo', 200, 1.0 )
                            )
        self.assertEquals( len( impatiens.listInvocations() ), 1 )
        self.assertEquals( len( impatiens.listErrors() ), 1 )
        assert not impatiens.listErrors( fatal_only=1 )
        assert not impatiens.listChildren()

    def test_composite( self ):

        top = Result( test=MARKER )

        assert top.getTest() is MARKER
        assert top.timeRequests()
        assert top.checkResponses()
        assert top.checkElapsedTimes()

        child = top.newChild( test=MARKER2 )

        assert child.getTest() is MARKER2
        self.assertEquals( child.timeRequests(), top.timeRequests() )
        self.assertEquals( child.checkResponses(), top.checkResponses() )
        self.assertEquals( child.checkElapsedTimes(), top.checkElapsedTimes() )

        child.addInvocation( FauxInvocation( 500, 3600.0 )
                            , FauxRequest( 'foo', 200, 1.0 )
                            )
        self.assertEquals( len( child.listInvocations() ), 1 )
        self.assertEquals( len( child.listErrors() ), 1 )
        self.assertEquals( len( child.listErrors( fatal_only=1 ) ), 1 )

        self.assertEquals( len( top.listInvocations() ), 1 )
        self.assertEquals( len( top.listInvocations( roll_up=0 ) ), 0 )
        self.assertEquals( len( top.listErrors() ), 1 )
        self.assertEquals( len( top.listErrors( roll_up=0 ) ), 0 )
        self.assertEquals( len( top.listErrors( fatal_only=1 ) ), 1 )
        self.assertEquals( len( top.listErrors( fatal_only=1, roll_up=0 ) ), 0 )
        self.assertEquals( len( top.listChildren() ), 1 )

        top = Result( test=MARKER
                    , application=[]
                    , defaults={ 'foo':0 }
                    , time_requests=0
                    , check_responses=0
                    , check_elapsed_times=0
                    )
        child = top.newChild( test=MARKER2 )
        assert child.getApplication() is top.getApplication()
        self.assertEquals( child.getDefaults(), top.getDefaults() )
        self.assertEquals( child.timeRequests(), top.timeRequests() )
        self.assertEquals( child.checkResponses(), top.checkResponses() )
        self.assertEquals( child.checkElapsedTimes(), top.checkElapsedTimes() )

    #
    #   2.4 unittest has these.
    #
    if not hasattr( unittest.TestCase, 'assertEquals' ):
        def assertEquals( self, lhs, rhs ):
            assert lhs == rhs, '%s == %s' % (lhs, rhs )

def test_suite():
    return unittest.makeSuite( ResultTests )

def run():
    unittest.TextTestRunner().run(test_suite())

if __name__ == '__main__':
    run()
